@@ -8,6 +8,7 @@ import os
 import urlparse
 import json
 import gevent
+from gevent.pool import Pool
 
 from .patterns import Regex
 from .responses import ( DoneLoad, DoneFind, Wait, MissingTags,
@@ -52,7 +53,10 @@ class Request(object):
 
 class Scraper(object):
 
-    def __init__(self, session=None, force_all=False):
+    def __init__(self, session=None, force_all=False, concurrency=5):
+        self._file_cache = dict()
+        self._pool = Pool(concurrency)
+
         if session is None:
             self._session = requests.Session()
         else:
@@ -76,7 +80,16 @@ class Scraper(object):
             if resolved_uri.scheme in ['http', 'https']:
                 instruction = json.loads(requests.get(resolved_uri).text)
             elif resolved_uri.scheme is '':
-                instruction = json.load(open(urlparse.urlunsplit(resolved_uri)))
+                resolved_uri_str = urlparse.urlunsplit(resolved_uri)
+
+                # Use our file cache if we can
+                instruction = self._file_cache.get(resolved_uri_str)
+
+                # Otherwise, load and save in the cache
+                if instruction is None:
+                    instruction = json.load(open(resolved_uri_str))
+                    self._file_cache[resolved_uri_str] = instruction
+
             else:
                 raise InvalidInstructionError("Reference to unsupported scheme '%s'" % (
                     resolved_uri.scheme))
@@ -135,75 +148,78 @@ class Scraper(object):
                 min_match.result, max_match.result))
 
         # Python counts a little differently
+        single_match = min_match == max_match
         max_match = None if max_match == -1 else max_match + 1
 
         # Default to regex as string
         name = name_sub.result if name_sub.result else None
-        #replace = replace_sub.result
 
         tag_match = tag_match_sub.result
 
         try:
             regex = Regex(find_sub.result, ignore_case, multiline, dot_matches_all,
                           replace_unsubbed)
-            # Negative max means we can't utilize the generator, sadly...
-            subs = [s for s in regex.substitutions(input)][min_match:max_match]
+
+            # This lets through max_match = None, which is OK for generator
+            if min_match > -1 and max_match > -1:
+                subs = regex.substitutions(input, min_match, max_match)
+            # Negative values mean we can't utilize the generator, sadly...
+            else:
+                subs = [s for s in regex.substitutions(input)][min_match:max_match]
+
+            greenlets = []
+            replaced_subs = []
+            # Call children once for each substitution, using it as input
+            # and with a modified set of tags.
+            for i, s_unsubbed in enumerate(subs):
+
+                fork_tags = copy.deepcopy(req.tags)
+
+                # Ensure we can use tag_match in children
+                if tag_match:
+                    fork_tags[tag_match] = str(i)
+
+                # Fail out if unable to replace.
+                s_sub = Substitution(s_unsubbed, fork_tags)
+                if s_sub.missing_tags:
+                    return MissingTags(req, s_sub.missing_tags)
+                else:
+                    s_subbed = s_sub.result
+                    replaced_subs.append(s_subbed)
+
+                # actually modify our available tags if it was 1-to-1
+                if single_match and name is not None:
+                    req.tags[name] = s_subbed
+
+                    # The tag_match name is chosen in instruction, so it's OK
+                    # to propagate it -- no pollution risk
+                    if tag_match:
+                        req.tags[tag_match] = str(i)
+
+                if name is not None:
+                    fork_tags[name] = s_subbed
+
+                child_scraper = Scraper(session=self._session, force_all=self._force_all)
+
+                greenlets.append(child_scraper.scrape_async(then,
+                                                            tags=fork_tags,
+                                                            input=s_subbed,
+                                                            uri=req.uri))
         except PatternError as e:
             return Failed(req, "'%s' failed because of %s" % (instruction['find'],
                                                               str(e)))
 
-        if len(subs) == 0:
+        if len(greenlets) == 0:
             return Failed(req, "No matches for '%s', evaluated to '%s'" % (
                 instruction['find'], find_sub.result))
 
-        greenlets = []
-        replaced_subs = []
-        # Call children once for each substitution, using it as input
-        # and with a modified set of tags.
-        for i, s_unsubbed in enumerate(subs):
-
-            fork_tags = copy.deepcopy(req.tags)
-
-            # Ensure we can use tag_match in children
-            if tag_match:
-                fork_tags[tag_match] = str(i)
-
-            # Fail out if unable to replace.
-            s_sub = Substitution(s_unsubbed, fork_tags)
-            if s_sub.missing_tags:
-                return MissingTags(req, s_sub.missing_tags)
-            else:
-                s_subbed = s_sub.result
-                replaced_subs.append(s_subbed)
-
-            # actually modify our available tags if it was 1-to-1
-            if len(subs) == 1 and name is not None:
-                req.tags[name] = s_subbed
-
-                # The tag_match name is chosen in instruction, so it's OK
-                # to propagate it -- no pollution risk
-                if tag_match:
-                    req.tags[tag_match] = str(i)
-
-            if name is not None:
-                fork_tags[name] = s_subbed
-
-            child_scraper = Scraper(session=self._session, force_all=self._force_all)
-
-            greenlets.append(child_scraper.scrape_async(then,
-                                                        tags=fork_tags,
-                                                        input=s_subbed,
-                                                        uri=req.uri))
-
         gevent.joinall(greenlets)
 
         # Build Results with responses from greenlets, substitute in tags
-        # (since replace was unsubbed)
         results = []
-        for i, s_unsubbed in enumerate(subs):
+        for i, replaced_sub in enumerate(replaced_subs):
             child_resps = greenlets[i].get()
-
-            results.append(Result(replaced_subs[i], child_resps))
+            results.append(Result(replaced_sub, child_resps))
 
         return DoneFind(req, name, description, results)
 
@@ -428,4 +444,4 @@ class Scraper(object):
         Scrape a request like `scrape`, except returns a greenlet which
         supplies the Response or list of Responses from `get`.
         """
-        return gevent.spawn(self.scrape, instruction, tags, input, force=False, **kwargs)
+        return self._pool.spawn(self.scrape, instruction, tags, input, force=False, **kwargs)
